@@ -9,12 +9,10 @@ class TaggedItemInline(admin.StackedInline):
     model = TaggedItem
 
 
+@admin.register(Tag)
 class TagAdmin(admin.ModelAdmin):
     inlines = [TaggedItemInline]
     list_display = ["name", "slug"]
     ordering = ["name", "slug"]
     search_fields = ["name"]
     prepopulated_fields = {"slug": ["name"]}
-
-
-admin.site.register(Tag, TagAdmin)
