@@ -3,5 +3,5 @@ from django.utils.translation import ugettext_lazy as _
 
 
 class TaggitAppConfig(BaseConfig):
-    name = 'taggit'
-    verbose_name = _('Taggit')
+    name = "taggit"
+    verbose_name = _("Taggit")
